==> audiochain-1.1.5-src.tgz: org/audiochain/devices/frequency/FftAnalyzerComponent.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.frequency;

import static java.lang.Math.*;
import static org.audiochain.devices.frequency.DigitalSignalProcessing.*;
import static org.audiochain.ui.gui.GlobalGuiContext.*;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JComponent;

import org.audiochain.model.AudioProject;


public class FftAnalyzerComponent extends JComponent {
	private static final long serialVersionUID = 1L;
	private static final Color transparent = new Color(0, 0, 0, 0);
	private double[] rmsMagnitudeFactors;
	private Color foreground = getGuiSettings().getForegroundColor();
	private Color lightBackground = getGuiSettings().getLightBackgroundColor();
	
	public FftAnalyzerComponent(final FftAnalyzerAudioDevice device) {
		AudioProject audioProject = device.getAudioProject();
		int frameRate = audioProject.getFrameRate();
		int sampleSize = audioProject.getSampleSizeInBits();
		device.addFftReaderListener(new IntervalMagnitudeFftAudioDataReaderListener(frameRate, sampleSize) {
			@Override
			public void fftReaderStart() {
				device.setFftLength(nextPowerOfTwo(getWidth()));
			}

			@Override
			public void fftReaderStop() {

			}

			@Override
			public void fftReaderClose() {
				rmsMagnitudeFactors = null;
				repaint();
			}

			@Override
			protected void update(double[] leftRmsMagnitudeFactor, double[] rightRmsMagnitudeFactor) {
				updateGraph(leftRmsMagnitudeFactor, rightRmsMagnitudeFactor);
			}
		});
		setOpaque(false);
		int h = 35;
		this.setMinimumSize(new Dimension(66, h));
		this.setPreferredSize(getMinimumSize());
		this.setMaximumSize(getMinimumSize());
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				device.setFftLength(nextPowerOfTwo(getWidth()));
			}
		});
	}

	private void updateGraph(double[] leftRmsMagnitudeFactor, double[] rightRmsMagnitudeFactor) {
		if (leftRmsMagnitudeFactor == rightRmsMagnitudeFactor) {
			rmsMagnitudeFactors = leftRmsMagnitudeFactor;
		} else {
			int n = leftRmsMagnitudeFactor.length;
			rmsMagnitudeFactors = new double[n];
			for (int i = 0; i < n; i++) {
				rmsMagnitudeFactors[i] = (leftRmsMagnitudeFactor[i] + rightRmsMagnitudeFactor[i]) / 2;
			}
		}
		repaint();
	}
	
	@Override
	public void paint(Graphics g) {
		double[] factors = this.rmsMagnitudeFactors;
		int w = getWidth();
		int h = getHeight();
		g.setColor(transparent);
		g.fillRect(0, 0, w, h);
		g.setColor(lightBackground);
		for (double i = 1; i > 0; i -= 0.1D) {
			double lg = -log(i);
			int y = (int) (lg * h);
			if (y < h) {
				g.drawLine(0, y, w, y);
			}
//			int x = (int) (w - (lg * w));
			int x = (int) (i * w);
			g.drawLine(x, 0, x, h);
		}
		g.setColor(lightBackground);
		g.drawRect(0, 0, w - 1, h - 1);
		if (factors != null) {
			int frequencies = factors.length;
			int horizontalPixelsPerFrequency = w / frequencies;
			if (horizontalPixelsPerFrequency <= 0) {
				horizontalPixelsPerFrequency = 1;
			}
			g.setColor(foreground);
			for (int i = 0, x = 0; i < frequencies; i++, x += horizontalPixelsPerFrequency) {
				double factor = factors[i];
				int fh = (int) round(factor * h);
				g.fillRect(x, h - fh, horizontalPixelsPerFrequency, fh);
			}
		}
	}
}

==> audiochain-1.1.6-src.tgz: org/audiochain/devices/frequency/FftAnalyzerComponent.java <==
/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.frequency;

import static java.lang.Math.log;
import static java.lang.Math.round;
import static org.audiochain.devices.frequency.DigitalSignalProcessing.nextPowerOfTwo;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JComponent;

import org.audiochain.model.AudioProject;
import org.audiochain.ui.gui.GuiSettings;


public class FftAnalyzerComponent extends JComponent {
	private static final long serialVersionUID = 1L;
	private static final Color transparent = new Color(0, 0, 0, 0);
	private double[] rmsMagnitudeFactors;
	private Color foreground = GuiSettings.getInstance().getForegroundColor();
	private Color lightBackground = GuiSettings.getInstance().getLightBackgroundColor();
	
	public FftAnalyzerComponent(final FftAnalyzerAudioDevice device) {
		AudioProject audioProject = device.getAudioProject();
		int frameRate = audioProject.getFrameRate();
		int sampleSize = audioProject.getSampleSizeInBits();
		device.addFftReaderListener(new IntervalMagnitudeFftAudioDataReaderListener(frameRate, sampleSize) {
			@Override
			public void fftReaderStart() {
				device.setFftLength(nextPowerOfTwo(getWidth()));
			}

			@Override
			public void fftReaderStop() {

			}

			@Override
			public void fftReaderClose() {
				rmsMagnitudeFactors = null;
				repaint();
			}

			@Override
			protected void update(double[] leftRmsMagnitudeFactor, double[] rightRmsMagnitudeFactor) {
				updateGraph(leftRmsMagnitudeFactor, rightRmsMagnitudeFactor);
			}
		});
		setOpaque(false);
		int h = 35;
		this.setMinimumSize(new Dimension(66, h));
		this.setPreferredSize(getMinimumSize());
		this.setMaximumSize(getMinimumSize());
		addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent e) {
				device.setFftLength(nextPowerOfTwo(getWidth()));
			}
		});
	}

	private void updateGraph(double[] leftRmsMagnitudeFactor, double[] rightRmsMagnitudeFactor) {
		if (leftRmsMagnitudeFactor == rightRmsMagnitudeFactor) {
			rmsMagnitudeFactors = leftRmsMagnitudeFactor;
		} else {
			int n = leftRmsMagnitudeFactor.length;
			rmsMagnitudeFactors = new double[n];
			for (int i = 0; i < n; i++) {
				rmsMagnitudeFactors[i] = (leftRmsMagnitudeFactor[i] + rightRmsMagnitudeFactor[i]) / 2;
			}
		}
		repaint();
	}
	
	@Override
	public void paint(Graphics g) {
		double[] factors = this.rmsMagnitudeFactors;
		int w = getWidth();
		int h = getHeight();
		g.setColor(transparent);
		g.fillRect(0, 0, w, h);
		g.setColor(lightBackground);
		for (double i = 1; i > 0; i -= 0.1D) {
			double lg = -log(i);
			int y = (int) (lg * h);
			if (y < h) {
				g.drawLine(0, y, w, y);
			}
//			int x = (int) (w - (lg * w));
			int x = (int) (i * w);
			g.drawLine(x, 0, x, h);
		}
		g.setColor(lightBackground);
		g.drawRect(0, 0, w - 1, h - 1);
		if (factors != null) {
			int frequencies = factors.length;
			int horizontalPixelsPerFrequency = w / frequencies;
			if (horizontalPixelsPerFrequency <= 0) {
				horizontalPixelsPerFrequency = 1;
			}
			g.setColor(foreground);
			for (int i = 0, x = 0; i < frequencies; i++, x += horizontalPixelsPerFrequency) {
				double factor = factors[i];
				int fh = (int) round(factor * h);
				g.fillRect(x, h - fh, horizontalPixelsPerFrequency, fh);
			}
		}
	}
}
